/* cris.h -- Values and opcodes (except the table) for the CRIS processor.
   Copyright (C) 1989, 1999 Free Software Foundation, Inc.

This file is part of GAS, GDB and the GNU binutils.

GAS/GDB/GNU binutils is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GAS/GDB/GNU binutils is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GAS or GDB or GNU binutils; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifndef __CRIS_H_INCLUDED_
#define __CRIS_H_INCLUDED_
#if !defined(__STDC__) && !defined(const)
#define const
#endif

/* REGISTERS */

/* Added 920330 by MA */
#define MAX_REG (15)
#define REG_SP  (14)
#define REG_PC  (15)

/* CPU version control of disassembly and assembly of instructions.
   May affect how the instruction is assembled, at least the size of
   immediate operands.  */
enum cris_insn_version_usage
{
  /* Any version. */
  cris_ver_version_all=0,

  /* Indeterminate (intended for disassembly only, or obsolete).  */
  cris_ver_warning,

  /* Simulator only (reserved).  */
  cris_ver_sim,

  /* Only for v0..3 (Etrax 1..4).  */
  cris_ver_v0_3,

  /* Only for v3 or higher (Etrax 1..4 and beyond).  */
  cris_ver_v3p,

  /* Only for v8 (Etrax 100).  */
  cris_ver_v8,

  /* Only for v8 or higher (Etrax 100, "NG").  */
  cris_ver_v8p,

  /* Only for v10 or higher ("Etrax NG").
     Of course some or all these of may change to cris_ver_v10p if/when
     there's a new revision. */
  cris_ver_v10p
};


/* Added 920330 by MA.  Hacked by H-P.  */
struct cris_spec_reg
{
  const char    * const name;
  unsigned int  number;

  /* If negative, it is a guess that should produce a warning.  */
  int reg_size;

  enum cris_insn_version_usage applicable_version;
  const char *const warning;
};

extern const struct cris_spec_reg cris_spec_regs[];

/* Definitions for condition codes */
/* Added 920319 by MA */
/* Updated 920401 by MA */

#define CC_CC  0x0
#define CC_HS  0x0
#define CC_CS  0x1
#define CC_LO  0x1
#define CC_NE  0x2
#define CC_EQ  0x3
#define CC_VC  0x4
#define CC_VS  0x5
#define CC_PL  0x6
#define CC_MI  0x7
#define CC_LS  0x8
#define CC_HI  0x9
#define CC_GE  0xA
#define CC_LT  0xB
#define CC_GT  0xC
#define CC_LE  0xD
#define CC_A   0xE
#define CC_EXT 0xF


/* Added 920406 by MA */
/* 920713  MA  Changed to bit within byte instead of word */
#define AUTOINCR_BIT (0x04)


/* 920324 MA added PC_RELATIVE */
/* 920508 MA added pcrel_type */

/* Used as pcrel parameter in fix_new calls */
enum cris_pcrel_type { NOT_PC_RELATIVE, PC_RELATIVE };

/* OPCODE-DEPENDENT CONSTANTS */

/* Updated 920508 by MA to Preliminary specification - CGARISC INTRUCTION SET Rev. 0.12 */


/* Prefixes */

/* Added 920402 by MA */
/* Changed 930408 by HP */
#define BDAP_QUICK_OPCODE  (0x0100)
#define BDAP_QUICK_Z_BITS  (0x0e00)

#define BIAP_OPCODE        (0x0540)
#define BIAP_Z_BITS        (0x0a80)

#define DIP_OPCODE         (0x0970)
#define DIP_Z_BITS         (0xf280)

/* Added 920326 by MA */
/* Changed 930408 by HP */
#define BDAP_INDIR_LOW     (0x40)
#define BDAP_INDIR_LOW_Z   (0x80)
#define BDAP_INDIR_HIGH    (0x09)
#define BDAP_INDIR_HIGH_Z  (0x02)

/* Not used, but this is how the above are used: /hp 930408 */
#define BDAP_INDIR_OPCODE  (BDAP_INDIR_HIGH * 0x0100 + BDAP_INDIR_LOW)
#define BDAP_INDIR_Z_BITS  (BDAP_INDIR_HIGH_Z * 0x100 + BDAP_INDIR_LOW_Z)
#define BDAP_PC_LOW        (BDAP_INDIR_LOW + REG_PC)
#define BDAP_INCR_HIGH     (BDAP_INDIR_HIGH + AUTOINCR_BIT)


/* Bcc quick */

/* Added 920713 by MA */
/* Changed 930408 by HP */
/* Don't forget to change Z_BITS if you change OPCODE !!! */
#define BRANCH_QUICK_Z_BITS  (0x0F00)
#define BRANCH_QUICK_LOW     (0)
#define BRANCH_QUICK_HIGH    (0)
#define BRANCH_QUICK_OPCODE  (BRANCH_QUICK_HIGH * 0x0100 + BRANCH_QUICK_LOW)


/* BA quick */

/* Added 920425 by MA */
/* 920713  MA  Defined in terms of the BRANCH_QUICK_HIGH constant */
#define BA_QUICK_HIGH (BRANCH_QUICK_HIGH + CC_A * 0x10)

/* Added 920702 by MA */
/* 920703  MA  Construct from low and high byte values */
#define BA_QUICK_OPCODE  (BA_QUICK_HIGH * 0x100 + BRANCH_QUICK_LOW)


/* Bcc [PC+] */

/* Added 920326 by MA */
/* Changed 930408 by HP */
#define BRANCH_PC_LOW            (0xFF)
#define BRANCH_INCR_HIGH         (0x0D)

/* Added 920425 by MA */
/* 920703  MA  Construct from low and high byte values */
#define BA_PC_INCR_OPCODE  ((BRANCH_INCR_HIGH + CC_A * 0x10) * 0x0100 + BRANCH_PC_LOW)


/* JUMP */

/* Added 920713 by MA */
/* Changed 930408 by HP */
/* Don't forget to change Z_BITS if you change OPCODE !!! */
/* Note that old versions generated special register 8 (in high bits)
   and not-that-old versions recognized it as a jump-instruction.
   That opcode now belongs to JUMPU.  */
#define JUMP_INDIR_OPCODE  (0x0930)
#define JUMP_INDIR_Z_BITS  (0xf2c0)

/* Added 920511 by MA */
/* 920713  MA  Expressed in terms of JUMP_INDIR_OPCODE */
#define JUMP_PC_INCR_OPCODE  (JUMP_INDIR_OPCODE + AUTOINCR_BIT * 0x0100 + REG_PC)


/* NOP */

/* Added 920713 by MA */
/* Changed 930408 by HP */
#define NOP_Z_BITS  (0xFAF0)

/* Added 920702 by MA */
/* Changed 930408 by HP */
/* Don't forget to change Z_BITS if you change OPCODE !!! */
#define NOP_OPCODE  (0x050F)

/* OPCODE TABLE */


/*
 * Structure of an opcode table entry.
 */

/* 920312 by MA : renamed sparc_opcode to cris_opcode, redeclared match and lose to unsigned int */
/* 920603 MA Added imm_oprnd_size field */
enum cris_imm_oprnd_size_type { SIZE_NONE, SIZE_FIX_32, SIZE_SPEC_REG, SIZE_FIELD };

struct cris_opcode
{
    const char    *name;
    unsigned int  match;     /* Bits that must be set.  */
    unsigned int  lose;      /* Bits that must not be set.  */
    const char    *args;     /* See the table in "opcodes/cris-opc.c".  */
    char          delayed;   /* Nonzero if this is a delayed branch */
                             /* instruction.                        */

    enum cris_imm_oprnd_size_type imm_oprnd_size;  /* Size of immediate operands */
    enum cris_insn_version_usage applicable_version;
};

extern const struct cris_opcode cris_opcodes[];
#endif /* __CRIS_H_INCLUDED_ */
